<?php
/*
+-----------------------------------------------------------------------------+
| (c) Copyright IBM Corporation 2006, 2007.                                   |
| All Rights Reserved.                                                        |
+-----------------------------------------------------------------------------+
| Licensed under the Apache License, Version 2.0 (the "License"); you may not |
| use this file except in compliance with the License. You may obtain a copy  |
| of the License at -                                                         |
|                                                                             |
|                   http://www.apache.org/licenses/LICENSE-2.0                |
|                                                                             |
| Unless required by applicable law or agreed to in writing, software         |
| distributed under the License is distributed on an "AS IS" BASIS, WITHOUT   |
| WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.            |
| See the License for the specific language governing  permissions and        |
| limitations under the License.                                              |
+-----------------------------------------------------------------------------+
| Author: Wangkai Zai                                                         |
|                                                                             |
+-----------------------------------------------------------------------------+
*/
require "SCA/Bindings/soap/ServiceDescriptionGenerator.php";

if ( ! class_exists('SCA_Bindings_message_ServiceDescriptionGenerator', false) ) {

    class SCA_Bindings_message_ServiceDescriptionGenerator
    {
        public function generate($service_description)
        {
            SCA::$logger->log( "Entering");

            try {
                if ( isset($_GET['wsdl']) ) {
                    /*if request url end with ?wsdl, generate a wsdl file
                    using the ServiceDescriptionGenerator from soap binding*/
                    $str = SCA_Bindings_soap_ServiceDescriptionGenerator::
                        generateDocumentLiteralWrappedWsdl($service_description);
                }else{
                    $str =  $this->generateMPD($service_description);
                }
                header('Content-type: text/xml');
                echo $str;
                SCA::$logger->log('Exiting having generated wsdl');
            }
            catch (SCA_RuntimeException $se )
            {
                echo $se . "\n" ;
            } catch( SDO_DAS_XML_FileException $e) {
                echo "{$e->getMessage()} in {$e->getFile()}";
            }
            return;
        }

         public static function generateMPD($service_desc)
         {

                //Get a DAS
                $xmldas = SDO_DAS_XML::create(dirname(__FILE__)."\MessageProviderDescription.xsd");
                $mpd_doc = $xmldas->createDocument();
                $msgProvider = $mpd_doc->getRootDataObject() ;

                foreach ($service_desc->binding_config as $key => $value){
                    if ( $key == "userid" || $key == "password") {
                        /*userid and password will not output to the description file*/
                        continue;
                    }
                    $msgProvider->$key=$value ;  
                }
    
                if (!isset($msgProvider->protocol)) {
                    throw new SCA_RuntimeException('"protocol" is mandatory for MS binding. ');
                }
                
                if (!isset($msgProvider->queueurl)) {
                    /*the default queue name is the service component's class name*/
                    $msgProvider->queueurl = 'queue://' . $service_desc->class_name;
                }
                $str = $xmldas->saveString($mpd_doc, 2);
            
            return $str;

         }

    }
}

?>